function [nextScore,nextPopulation,state] = stepGA(thisScore,thisPopulation,options,state,GenomeLength,FitnessFcn,isOn,time,L1,L2,L,temporary_vars, tech_lim)
%  L1=41; L2=6; % L=53;
%generation=state.Generation;
%global tech_lim
%STEPGA Moves the genetic algorithm forward by one generation
%   This function is private to GA.

%   Copyright 2003-2007 The MathWorks, Inc.

% how many crossover offspring will there be from each source?
nEliteKids = options.EliteCount;
nXoverKids = round(options.CrossoverFraction * (size(thisPopulation,1) - nEliteKids));
nMutateKids = size(thisPopulation,1) - nEliteKids - nXoverKids;
% how many parents will we need to complete the population?
nParents = 2 * nXoverKids + nMutateKids;

% decide who will contribute to the next generation

% fitness scaling
state.Expectation = feval(options.FitnessScalingFcn,thisScore,nParents,options.FitnessScalingFcnArgs{:});

% selection. parents are indices into thispopulation
parents = feval(options.SelectionFcn,state.Expectation,nParents,options,options.SelectionFcnArgs{:});

% shuffle to prevent locality effects. It is not the responsibility
% if the selection function to return parents in a "good" order so
% we make sure there is a random order here.
parents = parents(randperm(length(parents)));

[unused,k] = sort(thisScore);

% Everyones parents are stored here for genealogy display
state.Selection = [k(1:options.EliteCount);parents'];

% here we make all of the members of the next generation
eliteKids  = thisPopulation(k(1:options.EliteCount),:);
xoverKids  = feval(options.CrossoverFcn, parents(1:(2 * nXoverKids)),options,GenomeLength,FitnessFcn,thisScore,thisPopulation,options.CrossoverFcnArgs{:});
mutateKids = feval(options.MutationFcn,  parents((1 + 2 * nXoverKids):end), options,GenomeLength,FitnessFcn,state,thisScore,thisPopulation,options.MutationFcnArgs{:});

% group them into the next generation
nextPopulation = [ eliteKids ; xoverKids ; mutateKids ];


% score the population
% We want to add the vectorizer if fitness function is NOT vectorized
% if strcmpi(options.Vectorized, 'off')
% %    vectorizedFitnessFcn=fcnvectorizer(nextPopulation,FitnessFcn,1,options.SerialUserFcn);
%     nextScore = fcnvectorizer(nextPopulation,FitnessFcn,1,options.SerialUserFcn);
% else
%     nextScore = FitnessFcn(nextPopulation);
% end
% % Make sure score is a column vector
% nextScore = nextScore(:);


%% Added code %%

%     [nextScore,I]=sort(nextScore, 'ascend');
%     nextPopulation=nextPopulation(I,:);
% %     if rand<0.03
% %         newSolution=TurnOffMutation_I(nextPopulation(1,:), L1, time);
% %         [nextPopulation(1,:), nextScore(1)]=chooseBest(nextPopulation(1,:),nextScore(1),newSolution, FitnessFcn);
% %     end
% %     [nextPopulation(1,:), nextScore(1)]=IntelligentMutation_II(nextPopulation(1,:), nextScore(1), FitnessFcn, L1, time);
% %     [nextPopulation(1,:), nextScore(1)]=advancedWindowSwap(nextPopulation(1,:), nextScore(1),  FitnessFcn, L1, time);
%
%     parfor i=1:options.EliteCount
%         % Turn off generator mutation
%         if rand<0.03
%             newSolution=TurnOffMutation_I(nextPopulation(i,:), L1, time);
%             [nextPopulation(i,:), nextScore(i)]=chooseBest(nextPopulation(i,:),nextScore(i),newSolution, FitnessFcn);
%         end
%         %                 if rand<0.25
%         %                         [nextPopulation(i,:), nextScore(i)]=StorageHillClimb(nextPopulation(i,:), nextScore(i), FitnessFcn, L1, L2, time);
%         %                 end
%         %         Window swap mutation
%         %               [nextPopulation(i,:), nextScore(i)]=advancedWindowSwap(nextPopulation(i,:), nextScore(i),  FitnessFcn, L1, time);
%         %           Intelligent mutation
%         if true %rand<0.5
%             [nextPopulation(i,:), nextScore(i)]=IntelligentMutation_II(nextPopulation(i,:), nextScore(i), FitnessFcn, L1, time);
%         else
%             nextPopulation(i,:)=IntelligentMutation_I(nextPopulation(i,:), L1, time);
%             nextScore(i)=FitnessFcn(nextPopulation(i,:));
%         end
%     end
%     parfor i=options.EliteCount+1:options.PopulationSize
%         % Turn off generator mutation
%         if rand<0.03
%             nextPopulation(i,:)=TurnOffMutation_I(nextPopulation(i,:), L1, time);
%             nextScore(i)=FitnessFcn(nextPopulation(i,:));
%         end
%         % Intelligent mutation
%         if rand<0.5
%             [nextPopulation(i,:), nextScore(i)]=IntelligentMutation_II(nextPopulation(i,:), nextScore(i), FitnessFcn, L1, time);
%             %[nextPopulation(i,:), nextScore(i)]=chooseBest(nextPopulation(i,:),nextScore(i),newSolution, FitnessFcn);
%         else
%             nextPopulation(i,:)=IntelligentMutation_I(nextPopulation(i,:), L1, time);
%             nextScore(i)=FitnessFcn(nextPopulation(i,:));
%         end
%     end
%     % Stochastic hill climb on best solution
%     %        [nextPopulation(1,:), nextScore(1)]=stochasticHillClimb(nextPopulation(1,:), nextScore(1), FitnessFcn,L1, L, time);
nextScore=zeros(options.PopulationSize,1);

f=1;
nextPopulation=TurnOffMutation_v(nextPopulation, L1, time, options.PopulationSize,f,GenomeLength);

if isOn
    mutationType1=[false;rand(options.PopulationSize-1,1)<0.5];
else
    mutationType1=[false;true(options.PopulationSize-1,1)];
end

mutationType2=not(mutationType1);

nextPopulation(mutationType1,:)=IntelligentMutation_Iv(nextPopulation(mutationType1,:), L, time);
nextScore(mutationType1) = fcnvectorizer(nextPopulation(mutationType1,:),FitnessFcn,1,options.SerialUserFcn);
[nextPopulation(mutationType2,:),nextScore(mutationType2)]=IntelligentMutation_IIv(nextPopulation(mutationType2,:), FitnessFcn, options.SerialUserFcn,@fcnvectorizer, L, time);

if isOn
    l=200;
    [~,f]=min(nextScore);

    while f~=l
        l=f;
 %       [nextPopulation(f,:), nextScore(f)]=advancedWindowSwap(nextPopulation(f,:), nextScore(f), FitnessFcn, L1, time);
        [nextPopulation(l,:), nextScore(l)]=repair(nextPopulation(l,:), nextScore(l), FitnessFcn,L1,time,temporary_vars, tech_lim);
        [~,f]=min(nextScore);
    end
%[nextPopulation(f,:), nextScore(f)]=advancedWindowSwap(nextPopulation(f,:), nextScore(f), FitnessFcn, L1, time);
%     [nextPopulation(f,:), nextScore(f)]=StorageHillClimb(nextPopulation(f,:), nextScore(f), FitnessFcn, L1, L2, time);    
end

%%
state.FunEval = state.FunEval + size(nextScore,1);


