function [production, cost, new_stored_energy]=dispatch_s2(lb, ub, prices, power_demand, active,...
    charging_units, stored_energy, installed_power, storage_capacity, storage_power, L1, L2,...
    storageEfficiency, limitations)

numActive=sum(active);
v=1:numActive;


% Storage operation
I=-sum(charging_units(1:L2/2))+sum(charging_units(L2/2+1:L2)); % is charging=> e<0
isCharging=I<0;
isDischarging=I>0;
charge_limit=isCharging*-min(storage_capacity-stored_energy, storage_power);
discharge_limit=isDischarging*min(stored_energy*storageEfficiency, storage_power);

if numActive==0
    if power_demand>=0
        production=[zeros(1,L1), min(discharge_limit,power_demand)];
    else
        production=[zeros(1,L1), max(charge_limit,power_demand)];
    end
else
    
    % Operation of boilers
    max_power=[ub.*installed_power.*active,discharge_limit];
    min_power=[lb.*installed_power.*active,charge_limit];
    
    [sortedPrices, order]=sort(prices, 'descend');
    priceChange=[1,v(diff(sortedPrices(v))<0)+1,v(end)+1];
    
    difference=max_power-min_power;
    
    % 1. If limitations are violated, turn down required units!
    for i=1:length(limitations)
        [~,order2]=sort(prices(limitations(i).units), 'descend');
        for j=limitations(i).units(order2)
            o=sum(max_power(limitations(i).units))-limitations(i).maximum;
            if o<=0
                break;
            end
            max_power(j)=max_power(j)-min(difference(j), o);
            difference(j)=max_power(j)-min_power(j);
        end
    end
    
    production=max_power;
    overshoot=sum(production)-power_demand; % Find overshoot in production
    indexes=order(1:numActive);
    sortedDifference=difference(indexes);
    
    cOvershoot=ones(1,numActive)*-1000;
    
    % First iteration
    i=1;
    cOvershoot(i)=overshoot-sortedDifference(i);
    % end
    if cOvershoot(i)>=0 && numActive>=2
        for i=2:numActive
            cOvershoot(i)=cOvershoot(i-1)-sortedDifference(i);
            if (cOvershoot(i)<0 )
                break;
            end
        end
    end
    if isCharging
        positions=priceChange(i>=priceChange);
        position=positions(max(end+I,1));
        % equal for both
        indexes=[indexes(1:position-1),L1+1,indexes(position:end)];
        try
            a=cOvershoot(position-1);
        catch
            a=overshoot;
        end
        cOvershoot=[cOvershoot(1:position-1),a,cOvershoot(position:end)];
        %
        cOvershoot(position:i+1)=cOvershoot(position:i+1)+charge_limit;
    end
    if isDischarging
        a=circshift(i<priceChange,-1);
        a(end)=true;
        positions=priceChange(a);
        position=positions(min(end,I));
        % equal for both
        indexes=[indexes(1:position-1),L1+1,indexes(position:end)];
        try
            a=cOvershoot(position-1);
        catch
            a=overshoot;
        end
        cOvershoot=[cOvershoot(1:position-1),a,cOvershoot(position:end)];
        %
        last=min(i+1,numActive+1);
        cOvershoot(position:last)=cOvershoot(position:last)-discharge_limit;
    end
    
    setToMin=indexes(cOvershoot>=0);
    j=find(cOvershoot<0,1);
    production(setToMin)=min_power(setToMin);
    if ~isempty(j)
        setToLimit=indexes(j);
        production(setToLimit)=min_power(setToLimit)+min(-cOvershoot(j), difference(setToLimit));
    end
    % OLD
    % error=cOvershoot(i);
    % current_charge=0;
    % if error<-differences(i) % if too few units are enabled
    %     if error<-discharge_limit
    %         current_charge=discharge_limit;
    %         production(indexes(i))=min_power(indexes(i))+min(-cOvershoot(i)-discharge_limit, differences(i));
    %     else
    %         current_charge=-error;
    %     end
    %
    % else %if (e~=0 )
    %     production(indexes(i))=min_power(indexes(i))+min(-cOvershoot(i), differences(i)); % *(e<=0); % Too few units: all production on one unit.
    %     if (e<0 ) % is_charging=e<0;
    %         for j=i:-1:max(1,i+e+1)
    %             k=indexes(j);
    %             temp=production(k);
    %             production(k)=min(max_power(k), temp+(charge_limit+current_charge));
    %             current_charge=current_charge-production(k)+temp;
    %         end
    %     elseif (e~=0 )
    %         for j=i:min(numActive,i+e-1)
    %             k=indexes(j);
    %             temp=production(k);
    %             production(k)=max(min_power(k), temp-(discharge_limit-current_charge));
    %             current_charge=current_charge-production(k)+temp;
    %         end
    %     end
    % end
end

current_charge=production(end);
if isCharging
    new_stored_energy=stored_energy-current_charge;
else
    new_stored_energy=stored_energy-current_charge/storageEfficiency;
end
error=sum(production)-power_demand; %+current_charge
cost=production(1:L1)*transpose(prices)+(error<-0.01)*abs(error)*5000; % penalty= 5000;

