function [lin_prediction, neural_net_input, RMSE_lin, RMSE_nonlin, MAPE_lin, MAPE_nonlin, day_of_week]...
    =load_prediction(training_temp, training_power, training_time,...
    temp, power, time,yc)
% temp_data and power_data are recorded at the time time_data.
clc
%% Processing time data. Week of 168 hours. %%
[t_cat_season, t_weekend_day, t_day_of_week, t_hour_of_week, t_hour_of_day, t_cat_day_w, t_cat_hour_d,...
    t_cat_month, t_Y, t_M, t_D, t_H] =categorize_time(training_time);
[cat_season, weekend_day, day_of_week, hour_of_week, hour_of_day, cat_day_w, cat_hour_d...
   ,cat_month ,Y, M, D, H]=categorize_time(time);

%% Make weekly average data %%
[t_avg_temp, t_avg_power, t_avg_temp_indexed]=find_avg_of_week(training_temp, training_power, t_hour_of_week);
[avg_temp, ~, avg_temp_indexed]=find_avg_of_week(temp, power, hour_of_week);

%% Fitting with non linear generalized least squares %%
temp_power_fun=@(b, T_out)...
     b(1).*((T_out-b(2)).*0.5.*erfc((T_out-b(2))/(sqrt(2)*b(3)))-(b(3)/sqrt(2*pi))*exp(-(T_out-b(2)).^2/(2*b(3)^2)))+b(4);
[share, beta_week]=fitNonLin(temp_power_fun, training_temp, training_power, t_hour_of_week, t_avg_temp, t_avg_power);
[t_prediction]=predict_nonlin(temp_power_fun, share, beta_week, training_temp, t_hour_of_week, training_power);
[prediction]=predict_nonlin(temp_power_fun,share, beta_week, temp, hour_of_week, power);
% busday=isbusday(datestr(Time_2016)); % isbusday returns 1 if businessday

%% Creating categorical data table %%
[t_data, t_power_24h_ago]=create_table(training_power, training_temp, t_hour_of_week, t_hour_of_day...
    , t_day_of_week, t_M, t_prediction, t_avg_temp_indexed);
[data, power_24h_ago]=create_table(power, temp, hour_of_week, hour_of_day...
    , day_of_week, M, prediction, avg_temp_indexed);

%% Splitting data sets %%
% Split by temperature
T0=beta_week(2);
[t_set1, t_set2, ~]=split_data_set(t_data, T0);
[set1, set2, ~]=split_data_set(data, T0);

% Split by season
[winter, spring, summer, autumn, season_power] ...
    = split_by_season(power, temp, weekend_day, hour_of_day, cat_day_w, cat_hour_d, M, Y);
[t_winter, t_spring, t_summer, t_autumn, t_season_power] ...
    = split_by_season(training_power, training_temp, t_weekend_day, t_hour_of_day, t_cat_day_w, t_cat_hour_d, t_M, t_Y);

%% Linear least squares fitting %%
linfit1=fitlm(t_set1, 'power_data~temp_data*hour_of_week+temp_data*M+power_24h_ago', 'Robustopts', 'on') % , 'Robustopts', 'on')
linfit2=fitlm(t_set2, 'power_data~temp_data*hour_of_week+temp_data*M+power_24h_ago' , 'Robustopts', 'on')
linfit_of_nonlin=fitlm(t_data, 'power_data~nonlin_pred*hour_of_week+nonlin_pred*M+power_24h_ago', 'Robustopts', 'on')

RMSE_lin=1; RMSE_nonlin=1; MAPE_lin =1; MAPE_nonlin=1; R_lin=1; R_nonlin=1;

% lin_prediction = predict(linfit,data);
lin_prediction1 = predict(linfit1,set1); lin_prediction2 = predict(linfit2,set2);
lin_prediction1((isnan(lin_prediction1))&(not(set1.missing_data)))=0;
lin_prediction2((isnan(lin_prediction2))&(not(set2.missing_data)))=0;
lin_prediction=lin_prediction1+lin_prediction2;

RMSE_lin = sqrt(mean((lin_prediction - power).^2,'omitnan'));  % Root Mean Squared Error

MAPE_lin = 100*mean(abs((lin_prediction - power)./power),'omitnan');
count_n_lin=sum(isnan(lin_prediction));
 
 nonlin_prediction = predict(linfit_of_nonlin,data);
 RMSE_nonlin = sqrt(mean((nonlin_prediction - power).^2,'omitnan'));
 MAPE_nonlin = 100*mean(abs((nonlin_prediction - power)./power),'omitnan');
 count_n=sum(isnan(nonlin_prediction));
 
%% Neural network %%
%[winter, spring, summer, autumn]=useStruct2Cell(winter, spring, summer, autumn);
neural_net_input=[temp, weekend_day, cat_season];
%[Th, Tl]=find_of_day(training_temp, t_D);
 % epochwise back propagation through time (EBPTT)

%% Plotting %%
% plot (lin_prediction - power, '.')
% hold on
% plot (1:length(lin_prediction),zeros(1,length(lin_prediction)))
% hold off

% figure(1)
% gscatter(temp_data, power_data)
% xlabel('Temperature Ullevl [C]'); ylabel('Heat load [MW]'); title('Scatter Plot of Temperature vs Heat Load');
% hold on
% 
%  figure(3)
%  gscatter(t_avg_temp, t_avg_power); 
% xlabel('Average temperature [C]'); ylabel('Average heat load [MW]');
% title('Scatter Plot of Weekly Averages');
% hold on;
%  
% i=-20:30;
%  plot(i, temp_power_fun(beta_week,i), 'green')
%  hold off
%  
% week_nr=30;
max_y=0;
% [max_y,~,~]=plot_week(temp, power, day_of_week, week_nr, max_y, power);
% 
week_nr=2;
[max_y,~,~]=plot_week(temp, lin_prediction, day_of_week, week_nr, max_y,power);
[max_y,~,~]=plot_week(temp, nonlin_prediction, day_of_week, week_nr,max_y,power);
[max_y,~,~]=plot_week(temp(25:length(temp)-24), yc, day_of_week(25:length(temp)-24), week_nr,max_y, power(25:length(temp)-24));
legend('Heat load', 'Linear model prediction', 'Neural network prediction')
ylim([0 max_y]); 
hold off
title('Heat Load of Weeks 48 and 49, 2016')
